@@ -14,6 +14,9 @@ from mnemosyne.libmnemosyne.controller import Controller
 from mnemosyne.libmnemosyne.utils import remove_empty_dirs_in
 from mnemosyne.libmnemosyne.utils import expand_path, contract_path
 
+HOUR = 60 * 60 # Seconds in an hour.
+DAY = 24 * HOUR # Seconds in a day.
+
 
 class DefaultController(Controller):
 
@@ -60,6 +63,11 @@ class DefaultController(Controller):
             self.config().save()
             self.review_controller().reset()
             self.next_rollover = self.database().start_of_day_n_days_ago(n=-1)
+        #if time.time() > self.config()["last_db_maintenance"] + 90 * DAY:
+        #    print "TMP DEBUG: do db maintenance"
+            #self.component_manager.current("database_maintenance")\
+            #    (self.component_manager).activate()        
+        #    self.config()["last_db_maintenance"] = time.time()
 
     def update_title(self):
         title = _("Mnemosyne")
@@ -187,7 +195,7 @@ class DefaultController(Controller):
             self.review_controller().reset()
         return cards
 
-    def show_edit_card_dialog(self):
+    def show_edit_card_dialog(self): 
         self.stopwatch().pause()
         self.flush_sync_server()
         review_controller = self.review_controller()
@@ -283,6 +291,7 @@ class DefaultController(Controller):
                 # We've created a new fact here. Make sure the calling function
                 # has the information to reload the fact.
                 fact._id = new_cards[0].fact._id
+                fact.id = new_cards[0].fact.id
                 if is_currently_asked:
                     self.review_controller().card = new_cards[0]
                 return 0
@@ -592,8 +601,9 @@ _("'*.cards' files are not separate databases, but need to be imported in your c
             self.stopwatch().unpause()
             self.main_widget().show_information(\
                 _("The configuration database is not used to store cards."))
-            return            
-        if filename.startswith(os.path.join(data_dir, "backups")):
+            return
+        if os.path.normpath(filename).startswith(\
+            os.path.normpath(os.path.join(data_dir, "backups"))):
             result = self.main_widget().show_question(\
                 _("Do you want to replace your current database with one restored from this backup?\nNote that this will result in conflicts during the next sync, which need to be resolved by a full sync."),
                 _("Yes"), _("No"), "")
@@ -688,8 +698,8 @@ _("The configuration database cannot be used to store cards."))
         self.component_manager.current("compact_database_dialog")\
             (self.component_manager).activate()
         self.review_controller().reset_but_try_to_keep_current_card()
-        self.stopwatch().unpause()
-
+        self.stopwatch().unpause()       
+            
     def show_insert_img_dialog(self, filter):
 
         """Show a file dialog filtered on the supported filetypes, get a
