@@ -237,8 +237,9 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
         # effects from generating log events while syncing.
         self.syncing = False
         # For importing from a mnemosyne2 cards file, we need different side
-        # effect to be disabled/enabled.
+        # effects to be disabled/enabled.
         self.importing = False
+        self.importing_with_learning_data = False
 
     #
     # File operations.
@@ -281,7 +282,8 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
             return os.path.basename(self._path).\
                 split(self.database().suffix)[0]
 
-    def compact(self):
+    def defragment(self):
+        self.main_widget().set_progress_text(_("Defragmenting database..."))
         self.con.execute("vacuum")
         # Make sure the "Untagged" tag does not show up together with
         # different tags.
@@ -295,6 +297,7 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
                 self.con.execute(\
                     "delete from tags_for_card where _card_id=? and _tag_id=?",
                     (_card_id, untagged._id))
+        self.main_widget().close_progress()
 
     def new(self, path):
         self.unload()
@@ -329,22 +332,6 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
         self._current_criterion._tag_ids_active.add(tag._id)
         self.add_criterion(self._current_criterion)
 
-    def _activate_plugin_for_card_type(self, card_type_id):
-        found = False
-        for plugin in self.plugins():
-            for component in plugin.components:
-                if component.component_type == "card_type" and \
-                    component.id == card_type_id:
-                    found = True
-                    try:
-                        plugin.activate()
-                    except:
-                        raise RuntimeError, _("Error when running plugin:") \
-                            + "\n" + traceback_string()
-        if not found:
-            raise RuntimeError, _("Missing plugin for card type with id:") \
-                + " " + card_type_id
-
     def load(self, path):
         if self.is_loaded():
             self.unload()
@@ -375,9 +362,7 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
         # Upgrade.
         self.con.execute("""create index if not exists
             i_cards_3 on cards (_fact_id);""")
-        # Identify missing plugins for card types and their parents.
-        plugin_needed_ids = set()
-        builtin_ids = set(card_type.id for card_type in self.card_types())
+        # Activate all the plugins needed for all the card types.
         # Sometimes corruption keeps the global_variables table intact,
         # but not the cards table...
         try:
@@ -385,24 +370,12 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
                 self.con.execute("select distinct card_type_id from cards")]
         except:
             raise RuntimeError, _("Unable to load file.") + traceback_string()
+        # We also need to do this for the card types which are only defined
+        # and have no cards yet.
         defined_in_database_ids = [cursor[0] for cursor in \
             self.con.execute("select id from card_types")]
-        # Check if parents are missing plugins. We also need to do this for the
-        # defined ids, in case a card type has no cards.
         for id in used_ids + defined_in_database_ids:
-            while "::" in id: # Move up one level of the hierarchy.
-                id, child_name = id.rsplit("::", 1)
-                if id not in builtin_ids and id not in defined_in_database_ids:
-                    plugin_needed_ids.add(id)
-            if id not in builtin_ids and id not in defined_in_database_ids:
-                plugin_needed_ids.add(id)
-        for card_type_id in plugin_needed_ids:
-            try:
-                self._activate_plugin_for_card_type(card_type_id)
-            except RuntimeError, exception:
-                self._connection.close()
-                self._connection = None
-                raise exception
+            self.activate_plugins_for_card_type_with_id(id) 
         # Instantiate card types stored in this database. Since they could
         # depend on a plugin, the card types need to be instatiated last.
         for id in defined_in_database_ids:
@@ -415,7 +388,7 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
         for f in self.component_manager.all("hook", "after_load"):
             f.run()
         # We don't log the database load here, but in libmnemosyne.__init__,
-        # as we prefer to log the start of the program first.
+        # as we prefer to log the start of the program first.                          
 
     def save(self, path=None):
         # Update format.
@@ -784,6 +757,10 @@ _("Putting a database on a network drive is forbidden under Windows to avoid dat
                 break
             index += 1
         return result
+    
+    def has_tag_with_id(self, id):
+        return self.con.execute("select count() from tags where id=?",
+            (id, )).fetchone()[0] != 0
 
     #
     # Facts.
@@ -835,6 +812,10 @@ _("Putting a database on a network drive is forbidden under Windows to avoid dat
             (fact._id, ))
         self.log().deleted_fact(fact)
         del fact
+        
+    def has_fact_with_id(self, id):
+        return self.con.execute("select count() from facts where id=?",
+            (id, )).fetchone()[0] != 0        
 
     #
     # Cards.
@@ -883,6 +864,9 @@ _("Putting a database on a network drive is forbidden under Windows to avoid dat
             sql_res = self.con.execute(query + "_id=?", (id, )).fetchone()
         else:
             sql_res = self.con.execute(query + "id=?", (id, )).fetchone()
+        if sql_res is None or sql_res[3] is None:
+            from mnemosyne.libmnemosyne.utils import MnemosyneError
+            raise MnemosyneError
         fact = self.fact(sql_res[3], is_id_internal=True)
         # Note that for the card type, we turn to the component manager as
         # opposed to this database, as we would otherwise miss the built-in
@@ -954,9 +938,14 @@ _("Putting a database on a network drive is forbidden under Windows to avoid dat
                 _card_id) values(?,?)""", (tag._id, card._id))
 
     def delete_card(self, card, check_for_unused_tags=True):
-        self.con.execute("delete from cards where _id=?", (card._id, ))
-        self.con.execute("delete from tags_for_card where _card_id=?",
-            (card._id, ))
+        if card._id is None: 
+            # A card which was created and deleted before a sync, so that
+            # it has incomplete information.
+            self.con.execute("delete from cards where id=?", (card.id, ))
+        else:
+            self.con.execute("delete from cards where _id=?", (card._id, ))
+            self.con.execute("delete from tags_for_card where _card_id=?",
+                             (card._id, ))
         if not self.syncing and check_for_unused_tags:
             for tag in card.tags:
                 self.delete_tag_if_unused(tag)
@@ -1031,6 +1020,10 @@ _("Putting a database on a network drive is forbidden under Windows to avoid dat
                 object_id) values(?,?,?)""",
                 (EventTypes.EDITED_CARD, int(time.time()), card_id))
 
+    def has_card_with_id(self, id):
+        return self.con.execute("select count() from cards where id=?",
+            (id, )).fetchone()[0] != 0 
+    
     #
     # Fact views.
     #
@@ -1081,10 +1074,42 @@ _("Putting a database on a network drive is forbidden under Windows to avoid dat
             (fact_view.id, ))
         self.log().deleted_fact_view(fact_view)
         del fact_view
+        
+    def has_fact_view_with_id(self, id):
+        return self.con.execute("select count() from fact_views where id=?",
+            (id, )).fetchone()[0] != 0         
 
     #
     # Card types.
     #
+    
+    def activate_plugins_for_card_type_with_id(self, id):
+        builtin_ids = set(card_type.id for card_type in self.card_types())
+        defined_in_database_ids = [cursor[0] for cursor in \
+            self.con.execute("select id from card_types")]        
+        # Check if parents are missing plugins.
+        plugin_needed_ids = set()
+        while "::" in id: # Move up one level of the hierarchy.
+            id, child_name = id.rsplit("::", 1)
+            if id not in builtin_ids and id not in defined_in_database_ids:
+                plugin_needed_ids.add(id)
+        if id not in builtin_ids and id not in defined_in_database_ids:
+            plugin_needed_ids.add(id)
+        for card_type_id in plugin_needed_ids:
+            found = False
+            for plugin in self.plugins():
+                for component in plugin.components:
+                    if component.component_type == "card_type" and \
+                        component.id == card_type_id:
+                        found = True
+                        try:
+                            plugin.activate()
+                        except:
+                            raise RuntimeError, _("Error when running plugin:") \
+                                + "\n" + traceback_string()
+            if not found:
+                raise RuntimeError, \
+                _("Missing plugin for card type with id:") + " " + card_type_id      
 
     def add_card_type(self, card_type):
         self.con.execute("""insert into card_types(id, name,
@@ -1205,7 +1230,14 @@ _("Putting a database on a network drive is forbidden under Windows to avoid dat
             criterion.card_type_deleted(card_type)
             self.update_criterion(criterion)
         del card_type
-
+        
+    def has_card_type_with_id(self, id):
+        #if self.con.execute("select count() from card_types where id=?",
+        #    (id, )).fetchone()[0] != 0:
+        #    return True
+        #else: # It could be a built-in card type.
+        return id in [card_type.id for card_type in self.card_types()]
+    
     #
     # Criteria.
     #
@@ -1265,6 +1297,11 @@ _("Putting a database on a network drive is forbidden under Windows to avoid dat
     def criteria(self):
         return (self.criterion(cursor[0], is_id_internal=True) \
             for cursor in self.con.execute("select _id from criteria"))
+    
+    def has_criterion_with_id(self, id):
+        return self.con.execute("select count() from criteria where id=?",
+            (id, )).fetchone()[0] != 0 
+    
 
     #
     # Queries.
