@@ -370,26 +370,29 @@ class SQLiteLogging(object):
         w.close_progress()
         
     def archive_old_logs(self):
-        w = self.main_widget()
-        w.set_progress_text(_("Archiving old logs..."))
+        self.main_widget().set_progress_text(_("Archiving old logs..."))
+        self.backup()
+        one_year_ago = int(time.time()) - 356 * DAY
         # Create archive dir if needed.
         archive_dir = os.path.join(self.config().data_dir, "archive")
         if not os.path.exists(archive_dir):
             os.makedirs(archive_dir)
         # Create empty archive database.
         db_name = os.path.basename(self.database().path()).rsplit(".", 1)[0]
-        archive_name = db_name + "-" + \
-            datetime.datetime.today().strftime("%Y%m%d.db")            
+        archive_name = db_name + "-" + self.config().machine_id() + "-" +\
+            datetime.datetime.today().strftime("%Y%m%d-%H%M%S.db")            
         archive_path = os.path.join(archive_dir, archive_name)
         from mnemosyne.libmnemosyne.databases._sqlite3 import _Sqlite3
         arch_con = _Sqlite3(self.component_manager, archive_path)        
         from mnemosyne.libmnemosyne.databases.SQLite import SCHEMA
         arch_con.executescript(SCHEMA.substitute(pregenerated_data="")) 
+        arch_con.executescript("""drop index i_log_timestamp;
+                                  drop index i_log_object_id;""")        
         arch_con.commit()
         arch_con.close()
         # Transfer old logs.
-        script = string.Template("""
-            attach "$filename" as archive;
+        script = string.Template("""   
+            attach "$archive_path" as archive;
             begin; 
             insert into archive.log(event_type, timestamp, object_id, grade,
                 easiness, acq_reps, ret_reps, lapses, acq_reps_since_lapse,
@@ -399,13 +402,13 @@ class SQLiteLogging(object):
                 acq_reps, ret_reps, lapses, acq_reps_since_lapse,
                 ret_reps_since_lapse, scheduled_interval, actual_interval,
                 thinking_time, next_rep, scheduler_data from log 
-                    where timestamp<$timestamp; 
+                    where timestamp<$one_year_ago; 
             commit; 
             begin;
-            delete from log where timestamp>$timestamp;
+            delete from log where timestamp<$one_year_ago;
             end;
             vacuum;
-        """).substitute(_id=insertion_log_index, filename=filename)
+        """).substitute(archive_path=archive_path, one_year_ago=one_year_ago)
         self.con.executescript(script) 
-        w.close_progress()
+        self.main_widget().close_progress()
         
\ No newline at end of file
