@@ -7,6 +7,7 @@
 import os
 import re
 import time
+import sqlite3
 
 from openSM2sync.log_entry import LogEntry
 from openSM2sync.log_entry import EventTypes
@@ -15,9 +16,9 @@ from mnemosyne.libmnemosyne.tag import Tag
 from mnemosyne.libmnemosyne.fact import Fact
 from mnemosyne.libmnemosyne.card import Card
 from mnemosyne.libmnemosyne.translator import _
-from mnemosyne.libmnemosyne.utils import expand_path
 from mnemosyne.libmnemosyne.card_type import CardType
 from mnemosyne.libmnemosyne.fact_view import FactView
+from mnemosyne.libmnemosyne.utils import expand_path, MnemosyneError
 
 re_src = re.compile(r"""(src|data)=\"(.+?)\"""", re.DOTALL | re.IGNORECASE)
 
@@ -197,8 +198,13 @@ class SQLiteSync(object):
             """select object_id from log where event_type=? or
             event_type=?""", (EventTypes.ADDED_MEDIA_FILE,
             EventTypes.EDITED_MEDIA_FILE))]:
-            if os.path.exists(expand_path(filename, self.media_dir())):
-                filenames.add(filename)
+            try:
+                if os.path.exists(expand_path(filename, self.media_dir())):
+                    filenames.add(filename)
+            except UnicodeError:
+                # Workaround Android encoding issue.
+                if os.path.exists(expand_path(filename, self.media_dir()).encode("utf-8")):
+                    filenames.add(filename)                
         return filenames
 
     def generate_log_entries_for_settings(self):
@@ -245,8 +251,9 @@ class SQLiteSync(object):
                 if id in defined_in_database_ids and \
                     id not in active_card_type_ids:
                     parent_card_type_ids.add(id)
+        # Sort to make sure we add the parents first.
         active_objects["card_type_ids"] = \
-            active_card_type_ids.union(parent_card_type_ids)
+            sorted(active_card_type_ids.union(parent_card_type_ids))
         # Fact views.
         active_objects["fact_view_ids"] = []
         for card_type_id in active_objects["card_type_ids"]:
@@ -282,7 +289,7 @@ class SQLiteSync(object):
             log_entry["n_mem"] = sql_res[7]
             log_entry["act"] = sql_res[8]
         elif event_type in (EventTypes.ADDED_CARD, EventTypes.EDITED_CARD):
-            try:
+            if self.has_card_with_id(log_entry["o_id"]):
                 # Note that some of these values (e.g. the repetition count) we
                 # could in theory calculate from the previous state and the
                 # grade. However, we send the entire state of the card across
@@ -313,7 +320,7 @@ class SQLiteSync(object):
                 log_entry["sch_data"] = card.scheduler_data
                 if card.extra_data:
                     log_entry["extra"] = repr(card.extra_data)
-            except TypeError: # The object has been deleted at a later stage.
+            else: # The object has been deleted at a later stage.
                 pass
         elif event_type == EventTypes.REPETITION:
             log_entry["gr"] = sql_res[4]
@@ -329,14 +336,14 @@ class SQLiteSync(object):
             log_entry["n_rp"] = sql_res[14]
             log_entry["sch_data"] = sql_res[15]
         elif event_type in (EventTypes.ADDED_TAG, EventTypes.EDITED_TAG):
-            try:
+            if self.has_tag_with_id(log_entry["o_id"]):
                 tag = self.tag(log_entry["o_id"], is_id_internal=False)
                 if tag is None:
                     return None
                 log_entry["name"] = tag.name
                 if tag.extra_data:
                     log_entry["extra"] = repr(tag.extra_data)
-            except TypeError: # The object has been deleted at a later stage.
+            else: # The object has been deleted at a later stage.
                 pass
         elif event_type in (EventTypes.ADDED_MEDIA_FILE,
             EventTypes.EDITED_MEDIA_FILE, EventTypes.DELETED_MEDIA_FILE):
@@ -346,15 +353,15 @@ class SQLiteSync(object):
             if self.sync_partner_info.get("capabilities") == "cards":
                 # The accompanying ADDED_CARD and EDITED_CARD events suffice.
                 return None
-            try:
+            if self.has_fact_with_id(log_entry["o_id"]):
                 fact = self.fact(log_entry["o_id"], is_id_internal=False)
                 for fact_key, value in fact.data.iteritems():
                     log_entry[fact_key] = value
-            except TypeError: # The object has been deleted at a later stage.
+            else: # The object has been deleted at a later stage.
                 pass
         elif event_type in (EventTypes.ADDED_FACT_VIEW,
             EventTypes.EDITED_FACT_VIEW):
-            try:
+            if self.has_fact_view_with_id(log_entry["o_id"]):
                 fact_view = self.fact_view(log_entry["o_id"],
                     is_id_internal=False)
                 log_entry["name"] = fact_view.name
@@ -368,11 +375,9 @@ class SQLiteSync(object):
                 log_entry["type_answer"] = repr(fact_view.type_answer)
                 if fact_view.extra_data:
                     log_entry["extra"] = repr(fact_view.extra_data)
-            except TypeError: # The object has been deleted at a later stage.
-                pass
         elif event_type in (EventTypes.ADDED_CARD_TYPE,
             EventTypes.EDITED_CARD_TYPE):
-            try:
+            if self.has_card_type_with_id(log_entry["o_id"]):
                 card_type = self.card_type(log_entry["o_id"],
                     is_id_internal=False)
                 log_entry["name"] = card_type.name
@@ -388,18 +393,14 @@ class SQLiteSync(object):
                     repr(card_type.keyboard_shortcuts)
                 if card_type.extra_data:
                     log_entry["extra"] = repr(card_type.extra_data)
-            except TypeError: # The object has been deleted at a later stage.
-                pass
         elif event_type in (EventTypes.ADDED_CRITERION,
             EventTypes.EDITED_CRITERION):
-            try:
+            if self.has_criterion_with_id(log_entry["o_id"]):
                 criterion = self.criterion(log_entry["o_id"],
                     is_id_internal=False)
                 log_entry["name"] = criterion.name
                 log_entry["criterion_type"] = criterion.criterion_type
                 log_entry["data"] = criterion.data_to_sync_string()
-            except TypeError: # The object has been deleted at a later stage.
-                pass
         elif event_type == EventTypes.EDITED_SETTING:
             if log_entry["o_id"] not in self.config():  # Obsolete entry.
                 return None
@@ -408,9 +409,7 @@ class SQLiteSync(object):
 
     def add_tag_from_log_entry(self, log_entry):
         if self.importing:
-            already_imported = self.con.execute(\
-                "select count() from tags where id=?",
-                (log_entry["o_id"], )).fetchone()[0] != 0
+            already_imported = self.has_tag_with_id(log_entry["o_id"])
             same_name_in_database = self.con.execute(\
                 "select count() from tags where name=? and id!=?",
                 (log_entry["name"], log_entry["o_id"])).fetchone()[0] == 1
@@ -432,8 +431,16 @@ class SQLiteSync(object):
         # the object from the database. This is a bit slower than just filling
         # in harmless missing keys, but it is more robust against future
         # side effects of tag deletion.
-        if log_entry["type"] == EventTypes.DELETED_TAG:
-            return self.tag(log_entry["o_id"], is_id_internal=False)
+        if log_entry["type"] == EventTypes.DELETED_TAG:           
+            # Work around legacy logs which contain duplicate deletion events.
+            if self.has_tag_with_id(log_entry["o_id"]):
+                return self.tag(log_entry["o_id"], is_id_internal=False)
+            else:
+                # Leftover from old bug, should not reoccur.
+                self.main_widget().show_information(\
+            _("Deleting same tag twice during sync. Inform the developpers."))
+                log_entry["name"] = "irrelevant"
+                return Tag(log_entry["name"], log_entry["o_id"])                
         # If we are creating a tag that will be deleted at a later stage
         # during this sync, we are missing some (irrelevant) information
         # needed to properly create a tag object.
@@ -452,9 +459,7 @@ class SQLiteSync(object):
 
     def add_fact_from_log_entry(self, log_entry):
         if self.importing:
-            already_imported = self.con.execute(\
-                "select count() from facts where id=?",
-                (log_entry["o_id"], )).fetchone()[0] != 0
+            already_imported = self.has_fact_with_id(log_entry["o_id"])
             if already_imported:
                 log_entry["type"] = EventTypes.EDITED_FACT
                 return self.update_fact(\
@@ -462,6 +467,15 @@ class SQLiteSync(object):
         self.add_fact(self.fact_from_log_entry(log_entry))
 
     def fact_from_log_entry(self, log_entry):
+        # Work around legacy logs which contain duplicate deletion events.
+        # Leftover from old bug, should not reoccur.
+        if log_entry["type"] != EventTypes.ADDED_FACT and \
+           not self.has_fact_with_id(log_entry["o_id"]):
+            self.main_widget().show_information(\
+        _("Deleting same fact twice during sync. Inform the developpers."))                
+            fact = Fact({}, log_entry["o_id"])
+            fact._id = -1
+            return fact             
         # Get fact object to be deleted now.
         if log_entry["type"] == EventTypes.DELETED_FACT:
             return self.fact(log_entry["o_id"], is_id_internal=False)
@@ -473,14 +487,12 @@ class SQLiteSync(object):
         fact = Fact(fact_data, log_entry["o_id"])
         if log_entry["type"] != EventTypes.ADDED_FACT:
             fact._id = self.con.execute("select _id from facts where id=?",
-                (fact.id, )).fetchone()[0]
+                (fact.id, )).fetchone()[0]                
         return fact
 
     def add_card_from_log_entry(self, log_entry):
         if self.importing:
-            already_imported = self.con.execute(\
-                "select count() from cards where id=?",
-                (log_entry["o_id"], )).fetchone()[0] != 0
+            already_imported = self.has_card_with_id(log_entry["o_id"])
             if already_imported:
                 orig_card = self.card(log_entry["o_id"], is_id_internal=False)
                 card = self.card_from_log_entry(log_entry)
@@ -522,22 +534,16 @@ class SQLiteSync(object):
         # Get card object to be deleted now.
         if log_entry["type"] == EventTypes.DELETED_CARD:
             try:
-                # More future-proof version of code in the except statement.
-                # However, this will fail if after the last sync the other
-                # partner created and deleted this card, so that there is no
-                # fact information.
                 return self.card(log_entry["o_id"], is_id_internal=False)
-            except TypeError:
-                # Less future-proof version which just returns an empty shell.
-                # Make sure to set _id, though, as that will be used in
-                # actually deleting the card.
-                sql_res = self.con.execute("select _id from cards where id=?",
-                    (log_entry["o_id"], )).fetchone()
+            except MnemosyneError:  # There is no fact in the database.
+                # We have created and deleted this card since the last sync,
+                # so we just return an empty shell.                              
                 card_type = self.card_type_with_id("1")
                 fact = Fact({"f": "f", "b": "b"}, id="")
                 card = Card(card_type, fact, card_type.fact_views[0],
                     creation_time=0)
-                card._id = sql_res[0]
+                card._id = None # Signals special case to 'delete_card'.
+                card.id = log_entry["o_id"]
                 return card
         # Create an empty shell of card object that will be deleted later
         # during this sync.
@@ -555,17 +561,18 @@ class SQLiteSync(object):
         else:
             if log_entry["card_t"] not in \
                 self.component_manager.card_type_with_id:
-                # Try to activate a plugin for the card type. If this fails,
-                # it's possible that the data for this card type will follow
-                # later during the sync. In that case, create a dummy card
-                # type here, which will be corrected by a later edit event.
-                # Hovewer, make note that we still need to instantiate this
-                # card type later, so that we can catch errors, e.g. due to
-                # bad plugins.
+                # If the card type is not in the database, it's possible that
+                # the data for this card type will follow later during the 
+                # sync. In that case, create a dummy card type here, which 
+                # will be corrected by a later edit event. Hovewer, we still 
+                # need to instantiate this card type later, so that we can 
+                # catch errors, e.g. due to bad plugins.
                 try:
-                    self._activate_plugin_for_card_type(log_entry["card_t"])
-                    card_type = self.card_type_with_id(log_entry["card_t"])
-                except RuntimeError:
+                    self.activate_plugins_for_card_type_with_id\
+                        (log_entry["card_t"])
+                    card_type = self.card_type_with_id\
+                        (log_entry["card_t"])
+                except:
                     self.card_types_to_instantiate_later.add(\
                         log_entry["card_t"])
                     card_type = self.card_type_with_id("1")
@@ -574,7 +581,7 @@ class SQLiteSync(object):
                 card_type = self.card_type_with_id(log_entry["card_t"])
         fact = self.fact(log_entry["fact"], is_id_internal=False)
         # When importing, set the creation time to the current time.
-        if self.importing:
+        if self.importing and not self.importing_with_learning_data:
             log_entry["c_time"] = int(time.time())
             log_entry["m_time"] = int(time.time())
         for fact_view in card_type.fact_views:
@@ -583,9 +590,9 @@ class SQLiteSync(object):
                     creation_time=log_entry["c_time"])
                 break
         for tag_id in log_entry["tags"].split(","):
-            try:
+            if self.has_tag_with_id(tag_id):
                 card.tags.add(self.tag(tag_id, is_id_internal=False))
-            except TypeError:
+            else:
                 # The tag has been deleted later later during the log. Don't
                 # worry about it now, this will be corrected by a later
                 # EDITED_CARD event.
@@ -601,10 +608,10 @@ class SQLiteSync(object):
         # in the database functions.
         card.id = log_entry["o_id"]
         if (log_entry["type"] != EventTypes.ADDED_CARD) or self.importing:
-            try:
+            if self.has_card_with_id(card.id):
                 card._id = self.con.execute("select _id from cards where id=?",
                     (card.id, )).fetchone()[0]
-            except TypeError:
+            else:
                 # Importing a card for the first time, so it is not yet in the
                 # database.
                 pass
@@ -667,13 +674,18 @@ class SQLiteSync(object):
         self.log().edited_media_file(filename)
 
     def delete_media_file(self, log_entry):
-        filename = log_entry["fname"]
-        full_path = expand_path(filename, self.media_dir())
+        # Actually, we cannot take the responsibility to delete a media file,
+        # since it would e.g. break the corner case to add a media file, 
+        # delete it and than add it again.
+        pass
+    
+        #filename = log_entry["fname"]
+        #full_path = expand_path(filename, self.media_dir())
         # The file could have been remotely deleted before it got a chance to
         # be synced, so we need to check if the file exists before deleting.
-        if os.path.exists(full_path):
-            os.remove(full_path)
-        self.log().deleted_media_file(filename)
+        #if os.path.exists(full_path):
+        #    os.remove(full_path)
+        #self.log().deleted_media_file(filename)
 
     def add_fact_view_from_log_entry(self, log_entry):
         if self.importing:
@@ -688,9 +700,10 @@ class SQLiteSync(object):
                     self.fact_view_from_log_entry(log_entry))
         try:
             self.add_fact_view(self.fact_view_from_log_entry(log_entry))
-        except:
-            # Leftover from old bug.
-            print "Creating same fact view twice during sync."
+        except sqlite3.IntegrityError:
+            # Leftover from old bug, should not reoccur.
+            self.main_widget().show_information(\
+    _("Creating same fact view twice during sync. Inform the developpers."))
 
     def fact_view_from_log_entry(self, log_entry):
         # Get fact view object to be deleted now.
@@ -733,10 +746,13 @@ class SQLiteSync(object):
                 return self.update_card_type(\
                     self.card_type_from_log_entry(log_entry))
         try:
-            self.add_card_type(self.card_type_from_log_entry(log_entry))
-        except:
-            # Leftover from old bug.
-            print "Creating same card type twice during sync."
+            card_type = self.card_type_from_log_entry(log_entry)
+            self.activate_plugins_for_card_type_with_id(card_type.id)
+            self.add_card_type(card_type)
+        except sqlite3.IntegrityError:
+            # Leftover from old bug, should not reoccur.
+            self.main_widget().show_information(\
+    _("Creating same card type twice during sync. Inform the developpers."))            
 
     def card_type_from_log_entry(self, log_entry):
         # Get card type object to be deleted now.
@@ -748,6 +764,7 @@ class SQLiteSync(object):
             card_type = CardType(self.component_manager)
             card_type.id = log_entry["o_id"]
             card_type.fact_views = []
+            card_type.fact_keys_and_names = []
             return card_type
         # Create card type object.
         card_type = CardType(self.component_manager)
@@ -762,7 +779,7 @@ class SQLiteSync(object):
         card_type.required_fact_keys = eval(log_entry["required_fact_keys"])
         card_type.keyboard_shortcuts = eval(log_entry["keyboard_shortcuts"])
         if "extra" in log_entry:
-            card_type.extra_data = eval(log_entry["extra"])
+            card_type.extra_data = eval(log_entry["extra"])           
         return card_type
 
     def criterion_from_log_entry(self, log_entry):
@@ -801,11 +818,11 @@ class SQLiteSync(object):
             self.importing = True
         event_type = log_entry["type"]
         if "time" in log_entry:
-            self.log().timestamp = int(log_entry["time"])
+            self.log().timestamp = int(log_entry["time"]) 
         # TMP measure to allow syncing partners which did not yet store
         # machine ids for LOADED_DATABASE and SAVED_DATABASE.
         if not "o_id" in log_entry:
-            log_entry["o_id"] = ""
+            log_entry["o_id"] = ""        
         try:
             if event_type == EventTypes.STARTED_PROGRAM:
                 self.log().started_program(log_entry["o_id"])
@@ -867,7 +884,7 @@ class SQLiteSync(object):
                     self.reapply_default_criterion_needed = True
             elif event_type == EventTypes.DELETED_CRITERION:
                 self.delete_criterion(self.criterion_from_log_entry(log_entry))
-            elif event_type == EventTypes.EDITED_SETTING:
+            elif event_type == EventTypes.EDITED_SETTING:              
                 key, value = log_entry["o_id"], eval(log_entry["value"])
                 if key in self.config().keys_to_sync:
                     self.config()[key] = value
@@ -875,7 +892,7 @@ class SQLiteSync(object):
                         for render_chain in self.component_manager.\
                             all("render_chain"):
                             render_chain.renderer_for_card_type(card_type).\
-                                update(card_type)                    
+                                update(card_type)
                 else:
                     self.log_edited_setting(log_entry["time"], key)
         finally:
