@@ -144,11 +144,6 @@ class SM2Mnemosyne(Scheduler):
         self.stage = 1
         self.warned_about_too_many_cards = False
 
-    def heartbeat(self):
-        if time.time() > self._fact_ids_memorised_expires_at:
-            self._fact_ids_memorised = []
-            self._fact_ids_memorised_expires_at = int(time.time()) + DAY
-
     def set_initial_grade(self, cards, grade):
 
         """Sets the initial grades for a set of sister cards, making sure
