@@ -270,7 +270,8 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
         12 : _("You can resize columns in the card browser by dragging between the header labels."),
         15 : _("When editing or previewing cards from the card browser, PageUp/PageDown can be used to move to the previous/next card."),
         18 : _("You change the relative size of the card list, card type tree and tag tree by dragging the dividers between them."),
-        21 : _("In the search box, you can use SQL wildcards like _ (matching a single character) and % (matching one or more characters).")}
+        21 : _("In the search box, you can use SQL wildcards like _ (matching a single character) and % (matching one or more characters)."),
+        24 : _("Cards with strike-through text are inactive in the current set.")}
 
     def __init__(self, component_manager):
         BrowseCardsDialog.__init__(self, component_manager)
@@ -334,7 +335,7 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
         self.card_type_tree_wdgt.tree_wdgt.\
             itemClicked.connect(self.update_filter)
         self.tag_tree_wdgt.tree_wdgt.\
-            itemClicked.connect(self.update_filter)
+            itemClicked.connect(self.update_filter)        
         # Context menu.
         self.table.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
         self.table.customContextMenuRequested.connect(self.context_menu)
@@ -453,6 +454,7 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
         # Here, we don't unload the database already by ourselves, but leave
         # it to the edit dialog to only do so if needed.
         self.edit_dlg.before_apply_hook = self.unload_qt_database
+        self.edit_dlg.after_apply_hook = None
         self.edit_dlg.page_up_down_signal.connect(self.page_up_down_edit)
         if self.edit_dlg.exec_() == QtGui.QDialog.Accepted:
             self.card_type_tree_wdgt.rebuild()
@@ -471,6 +473,12 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
         self.table.selectRow(current_row + shift)
         _card_ids = self._card_ids_from_selection()
         card = self.database().card(_card_ids.pop(), is_id_internal=True)
+        self.edit_dlg.before_apply_hook = self.unload_qt_database
+        def after_apply():
+            self.load_qt_database()
+            self.display_card_table()
+        self.edit_dlg.after_apply_hook = after_apply 
+        self.edit_dlg.apply_changes()
         self.edit_dlg.set_new_card(card)
 
     def menu_preview(self):
