@@ -19,7 +19,8 @@ class CloneCardTypeDlg(QtGui.QDialog, Ui_CloneCardTypeDlg, Component):
             | QtCore.Qt.WindowMinMaxButtonsHint)
         self.setWindowFlags(self.windowFlags() \
             & ~ QtCore.Qt.WindowContextHelpButtonHint)
-        for card_type in self.database().sorted_card_types():
+        self.sorted_card_types = self.database().sorted_card_types()
+        for card_type in self.sorted_card_types:
             self.parent_type.addItem(_(card_type.name))
 
     def name_changed(self):
@@ -29,7 +30,7 @@ class CloneCardTypeDlg(QtGui.QDialog, Ui_CloneCardTypeDlg, Component):
             self.OK_button.setEnabled(True)
 
     def accept(self):
-        parent_instance = self.card_types()[self.parent_type.currentIndex()]
+        parent_instance = self.sorted_card_types[self.parent_type.currentIndex()]
         clone_name = unicode(self.name.text())
         clone = self.controller().clone_card_type(\
             parent_instance, clone_name)
