@@ -29,7 +29,6 @@ class PyQtConfiguration(Hook):
              "edit_card_dlg_state": None,
              "plugins_dlg_state": None,
              "clone_help_shown": False,
-             "compact_help_shown": False,
              "browse_cards_dlg_state": None,
              "browse_cards_dlg_splitter_1_state": None,
              "browse_cards_dlg_splitter_2_state": None,
@@ -47,6 +46,7 @@ class PyQtConfiguration(Hook):
              "password_for_sync_as_client": "",
              "started_add_edit_cards_n_times": 0,
              "started_browse_cards_n_times": 0,
+             "started_activate_cards_n_times": 0,
              "showed_help_on_renaming_sets": False,
              "showed_help_on_double_clicking_sets": False,
              "showed_help_on_adding_tags": False
