@@ -4,6 +4,8 @@
 
 from PyQt4 import QtCore, QtGui, QtWebKit
 
+import sys
+
 from mnemosyne.libmnemosyne.translator import _
 from mnemosyne.pyqt_ui.ui_review_wdgt import Ui_ReviewWdgt
 from mnemosyne.libmnemosyne.ui_components.review_widget import ReviewWidget
@@ -179,10 +181,15 @@ class QAOptimalSplit(object):
         self.answer.setHtml(self.empty())
 
 
+
 class ReviewWdgt(QtGui.QWidget, QAOptimalSplit, Ui_ReviewWdgt, ReviewWidget):
 
     auto_focus_grades = True
     number_keys_show_answer = True
+
+    key_to_grade_map = {QtCore.Qt.Key_QuoteLeft: 0, QtCore.Qt.Key_0: 0,
+            QtCore.Qt.Key_1: 1, QtCore.Qt.Key_2: 2, QtCore.Qt.Key_3: 3,
+            QtCore.Qt.Key_4: 4, QtCore.Qt.Key_5: 5}
      
     def __init__(self, component_manager):
         ReviewWidget.__init__(self, component_manager)
@@ -215,7 +222,15 @@ class ReviewWdgt(QtGui.QWidget, QAOptimalSplit, Ui_ReviewWdgt, ReviewWidget):
         parent.add_to_status_bar(self.notmem)
         parent.add_to_status_bar(self.act)
         parent.status_bar.setSizeGripEnabled(0)
-        QAOptimalSplit.__init__(self)
+        self.widget_with_last_selection = self.question
+        self.question.selectionChanged.connect(self.selection_changed_in_q)
+        self.answer.selectionChanged.connect(self.selection_changed_in_a)
+        QAOptimalSplit.__init__(self) 
+        self.mplayer = QtCore.QProcess()
+        self.media_queue = []
+        
+    def deactivate(self):
+        self.stop_media()
 
     def changeEvent(self, event):
         if hasattr(self, "show_button"):
@@ -229,17 +244,14 @@ class ReviewWdgt(QtGui.QWidget, QAOptimalSplit, Ui_ReviewWdgt, ReviewWidget):
         QtGui.QWidget.changeEvent(self, event)
 
     def keyPressEvent(self, event):
-        if not event.isAutoRepeat() and event.key() in \
-            [QtCore.Qt.Key_QuoteLeft, QtCore.Qt.Key_0, QtCore.Qt.Key_1,
-            QtCore.Qt.Key_2, QtCore.Qt.Key_3, QtCore.Qt.Key_4,
-            QtCore.Qt.Key_5] and \
-            self.review_controller().is_question_showing():
+        if event.key() in self.key_to_grade_map and not event.isAutoRepeat():
+            # Use controller function rather than self.is_answer_showing to
+            # deal with the map card type.
+            if self.review_controller().is_question_showing():
                 if self.number_keys_show_answer:
                     self.show_answer()
-        elif not event.isAutoRepeat() and event.key() \
-            == QtCore.Qt.Key_QuoteLeft and \
-            not self.review_controller().is_question_showing():
-            self.review_controller().grade_answer(0)
+            else:
+                self.grade_answer(self.key_to_grade_map[event.key()])   
         elif event.key() == QtCore.Qt.Key_PageDown:
             self.scroll_down()
         elif event.key() == QtCore.Qt.Key_PageUp:
@@ -296,14 +308,16 @@ class ReviewWdgt(QtGui.QWidget, QAOptimalSplit, Ui_ReviewWdgt, ReviewWidget):
         y -= int(0.9*(frame.geometry().height()))
         #frame.scroll(x, y)  # Seems buggy 20111121.
         frame.evaluateJavaScript("window.scrollTo(%d, %d);" % (x, y))
-
+        
+    def selection_changed_in_q(self):
+        self.widget_with_last_selection = self.question
+        
+    def selection_changed_in_a(self):
+        self.widget_with_last_selection = self.answer
+        
     def copy(self):
-        if self.review_controller().is_question_showing() or \
-           self.review_controller().card.fact_view.a_on_top_of_q:
-            webview = self.question
-        else:
-            webview = self.answer
-        webview.pageAction(QtWebKit.QWebPage.Copy).trigger()
+        self.widget_with_last_selection.pageAction(\
+            QtWebKit.QWebPage.Copy).trigger()
 
     def show_answer(self):     
         self.review_controller().show_answer()
@@ -379,6 +393,42 @@ class ReviewWdgt(QtGui.QWidget, QAOptimalSplit, Ui_ReviewWdgt, ReviewWidget):
         self.sched.setText(_("Scheduled: %d ") % scheduled_count)
         self.notmem.setText(_("Not memorised: %d ") % non_memorised_count)
         self.act.setText(_("Active: %d ") % active_count)
+        
+    def play_media(self, filename, start=None, stop=None):
+        if start is None:
+            start = 0
+        if stop is None:
+            stop = 999999
+        self.media_queue.append((filename, start, stop))
+        if self.mplayer.state() != QtCore.QProcess.Running:
+            self.play_next_file()
+            
+    def play_next_file(self):
+        filename, start, stop = self.media_queue.pop(0)
+        duration = stop - start
+        if duration > 400:
+            duration -= 300 # Compensate for mplayer overshoot. 
+        self.mplayer = QtCore.QProcess()
+        self.mplayer.finished.connect(self.done_playing)
+        if sys.platform == "win32":            
+            command = "mplayer.exe -slave -ao win32 -quiet \"" + filename + \
+                "\" -ss " + str(start) + " -endpos " + str(duration) 
+        elif sys.platform == "darwin":
+            command = "mplayer -slave -ao coreaudio -quiet \"" + filename + \
+                "\" -ss " + str(start) + " -endpos " + str(duration)
+        else:
+            command = "mplayer -slave -quiet \"" + filename + \
+                "\" -ss " + str(start) + " -endpos " + str(duration)
+        self.mplayer.start(command)
+            
+    def done_playing(self, result):
+        if len(self.media_queue) >= 1:
+            self.play_next_file()
+        
+    def stop_media(self):
+        if self.mplayer is not None:
+            self.mplayer.write("quit\n");
+        self.media_queue = []
 
     def redraw_now(self):
         self.repaint()
