@@ -135,11 +135,12 @@ class Client(Partner):
             if self.check_for_edited_local_media_files:
                 self.ui.set_progress_text("Checking for edited media files...")
                 self.database.check_for_edited_media_files()
-                self.ui.set_progress_text("Dynamically creating media files...")
-                self.database.dynamically_create_media_files()
+            # Always create media files, otherwise they are not synced across.
+            self.ui.set_progress_text("Dynamically creating media files...")
+            self.database.dynamically_create_media_files()
             # Set timeout long enough for e.g. a slow NAS waking from 
             # hibernation.
-            socket.setdefaulttimeout(45)
+            socket.setdefaulttimeout(60)
             self.login(username, password)
             # Generating media files at the server side could take some time,
             # so we update the timeout.
@@ -215,6 +216,11 @@ class Client(Partner):
                 if backup_file:
                     self.database.restore(backup_file)
         finally:
+            
+            # TMP: preparation for future fix
+            #self.get_sync_cancel()
+            
+            
             if self.con:
                 self.con.close()
             self.ui.close_progress()
@@ -339,7 +345,14 @@ class Client(Partner):
         elif self.server_info["user_id"] != client_info["user_id"] and \
             self.server_info["is_database_empty"] == False:
             raise SyncError("Error: mismatched user ids.\n" + \
-                "The first sync should happen on an empty database.")
+                "The first sync should happen on an empty database.\n" + \
+                "Backup then delete the local database and try again.")
+        if self.server_info["database_version"] != client_info["database_version"]:
+            raise SyncError("Error: database version mismatch.\n" + \
+                "Upgrade client and server to latest version.")
+        if self.server_info["program_version"] != client_info["program_version"]:
+            raise SyncError("Error: Mnemosyne version mismatch.\n" + \
+                "Upgrade client and server to latest version.")           
         self.database.create_if_needed_partnership_with(\
             self.server_info["machine_id"])
         self.database.merge_partners(self.server_info["partners"])
@@ -573,9 +586,9 @@ class Client(Partner):
     def get_sync_cancel(self):
         self.ui.set_progress_text("Cancelling sync...")
         self.request_connection()
+        session_token = self.server_info.get("session_token", "none")
         self.con.request("GET", self.url("/sync_cancel?session_token=%s" \
-            % (self.server_info["session_token"], )),
-            headers={"connection": "close"})
+            % (session_token, )), headers={"connection": "close"})          
         self._check_response_for_errors(self.con.getresponse())
 
     def get_sync_finish(self):
